#include "Vehicle.h"


/*The following is the declarations of date structures which are used in the flight software*/

/* Block signals: The local variables of the software to store temporary values and intermediate calculations*/
B_Vehicle_T Vehicle_B;

/* Block states: The state variables, that are required to store the values between different cycles*/
DW_Vehicle_T Vehicle_DW;

/* External inputs: The inputs of the flight software */
ExtU_Vehicle_T Vehicle_U;

/* External outputs: The outputs of the flight software */
ExtY_Vehicle_T Vehicle_Y;

/*Constants: These values are initialized in the start to store the constants*/
ConstB_Vehicle_T Vehicle_ConstB;




/*The entry point of the flight software. Simulink s-function calls this subroutine to execute the flight software at each time stamp
 * Inputs: const double B_mgm[3],const  double q_eci2b[4],const double xSunECI[3],	const  double qECEF2b[4],const double vECEF[3],
 * const double xECEF[3],const double mode,unsigned int RadiationNoiseFlag, unsigned int NumberOfBitFlipEvents
 * Outputs: double Commands[3], double Bbody[3], double AttitudeError[3]
 * */
void FlightSoftware(const double B_mgm[3],const  double q_eci2b[4],const double xSunECI[3],
					const  double qECEF2b[4],const double vECEF[3],const double xECEF[3],
					const double mode, double Commands[3], double Bbody[3], unsigned int RadiationNoiseFlag, unsigned int NumberOfBitFlipEvents, double AttitudeError[3])
{

	static int IdxState = 0;
	unsigned int index = 0;

	/* Initialization of structures need to be done only once*/
	if (IdxState == 0){
	Vehicle_initialize();
	IdxState = 1;}

	 /*Reading  external inputs */
	  Vehicle_U.Bmgm[0] = B_mgm[0];
	  Vehicle_U.Bmgm[1] = B_mgm[1];
	  Vehicle_U.Bmgm[2] = B_mgm[2];
	  Vehicle_U.q_eci2b[0] = q_eci2b[0];
	  Vehicle_U.q_eci2b[1] = q_eci2b[1];
	  Vehicle_U.q_eci2b[2] = q_eci2b[2];
	  Vehicle_U.q_eci2b[3] = q_eci2b[3];
	  Vehicle_U.xSunECI[0] = xSunECI[0];
	  Vehicle_U.xSunECI[1] = xSunECI[1];
	  Vehicle_U.xSunECI[2] = xSunECI[2];
	  Vehicle_U.qECEF2b[0] = qECEF2b[0];
	  Vehicle_U.qECEF2b[1] = qECEF2b[1];
	  Vehicle_U.qECEF2b[2] = qECEF2b[2];
	  Vehicle_U.qECEF2b[3] = qECEF2b[3];
	  Vehicle_U.vECEF[0] = vECEF[0];
	  Vehicle_U.vECEF[1] = vECEF[1];
	  Vehicle_U.vECEF[2] = vECEF[2];
	  Vehicle_U.xECEF[0] = xECEF[0];
	  Vehicle_U.xECEF[1] = xECEF[1];
	  Vehicle_U.xECEF[2] = xECEF[2];
	  Vehicle_U.Mode = mode;

	   /* Single Event Upset Function */
	   if (RadiationNoiseFlag == 1){
	 	  for(index = 0; index <NumberOfBitFlipEvents; index++)
		   SEU(); // SEU() will be called the number of times defined by NumberOfBitFlipEvents
	   }

	   /*Calling main function*/
	   Vehicle_step();

	   /* external outputs Mapping*/
	   Commands[0]=  (real_T)Vehicle_Y.Commands[0]  ;
	   Commands[1]= (real_T) Vehicle_Y.Commands[1] ;
	   Commands[2]= (real_T) Vehicle_Y.Commands[2] ;
	   Bbody[0]= (real_T) Vehicle_Y.Bbody[0] ;
	   Bbody[1]=  (real_T)Vehicle_Y.Bbody[1] ;
	   Bbody[2]=  (real_T)Vehicle_Y.Bbody[2] ;
	   AttitudeError[0] = (real_T)Vehicle_B.VectorConcatenate_a[0];
	   AttitudeError[1] = (real_T)Vehicle_B.VectorConcatenate_a[1];
	   AttitudeError[2] = (real_T)Vehicle_B.VectorConcatenate_a[2];
}






