#include <stdio.h>

#ifndef __SingleData_h__ 
#define __SingleData_h__ 

//#include "platform.h"

#include "WHO.h"

namespace HardenedData
{
	// enum WHO;
	template <class DataType> class SingleData;
}

namespace HardenedData
{
	template <class DataType> class SingleData
	{
		private: DataType d1; 


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		operator DataType() const {
			return d1;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType>& operator=(const HardenedData::SingleData<DataType>& val) {
			d1 = val.d1;
			return *this;  // Return a reference to myself
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType>& operator=(const DataType val) {
			d1 = val;
			return *this;  // Return a reference to myself
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType> operator+(const HardenedData::SingleData<DataType>& val) const {
			SingleData<DataType> result = *this;     // Make a copy of myself.  Same as MyClass result(*this);
			result.d1 += val.d1;       // Use += to add other to the copy.
			return result;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType> operator+(const DataType& val) const {
			SingleData<DataType> result = *this;     // Make a copy of myself.  Same as MyClass result(*this);
			result.d1 += val;       // Use += to add other to the copy.
			return result;
		}


		/// <summary>
		/// Overwrites the post-increment operator a++, when a is redundant TripleData<DataType>.
		/// It individually increments each replica of the left-hand variable a and returns the incremented value as a redundant TripleData<DataType>. The left-hand operator is therefore modified.
		/// When using this operator, the left-hand variable is not verified to be correct; each of the three replicas of the left-hand value are incremented separately, for a faster execution. As a consequence, incrementing a corrupted left-hand value, leaves the value corrupted.
		/// To have a correct value after incrementing, one shall explicitly use the sync() : DataType operator:
		///         TripleData<DataType> a;
		///         a++;
		///         a.sync();
		/// where sync() : DataType restores a correct value into a.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType>& operator++(int dummy) {
			d1++;
			return *this;  // Return a reference to myself
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType> operator++() {
			SingleData<DataType> temp;
			temp = (*this);
			d1++;
			return temp;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType>& operator+=(const HardenedData::SingleData<DataType>& val) {
			d1 += val.d1;
			return *this;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType>& operator+=(const DataType& val) {
			d1 += val;
			return *this;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType> operator-(const HardenedData::SingleData<DataType>& val) const {
			SingleData<DataType> result = *this;     // Make a copy of myself.  Same as MyClass result(*this);
			result.d1 -= val.d1;       // Use -= to add other to the copy.
			return result;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType> operator-(const DataType& val) const {
			SingleData<DataType> result = *this;     // Make a copy of myself.  Same as MyClass result(*this);
			result.d1 -= val;       // Use -= to add other to the copy.
			return result;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType>& operator--() {
			d1--;
			return *this;
		}


		/// <summary>
		/// Overwrites the post-decrement operator a--, when a is redundant SingleData<DataType>.
		/// It individually decrements each replica of the left-hand variable a and returns the incremented value as a redundant SingleData<DataType>. The left-hand operator is therefore modified.
		/// When using this operator, the left-hand variable is not verified to be correct; each of the three replicas of the left-hand value are decremented separately, for a faster execution. As a consequence, decrementing a corrupted left-hand value, leaves the value corrupted.
		/// To have a correct value after decrementing, one shall explicitly use the sync() : DataType operator:
		///         SingleData<DataType> a;
		///         a++;
		///         a.sync();
		/// where sync() : DataType restores a correct value into a.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType>& operator--(int dummy) {
			d1--;
			return *this;  // Return a reference to myself
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType>& operator-=(const HardenedData::SingleData<DataType>& val) {
			d1 -= val.d1;
			return *this;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType>& operator-=(const DataType& val) {
			d1 -= val;
			return *this;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType> operator*(const HardenedData::SingleData<DataType>& val) const {
			SingleData<DataType> result = *this;     // Make a copy of myself.  Same as MyClass result(*this);
			result.d1 *= val.d1;       // Use *= to multiply other to the copy.
			return result;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType> operator*(const DataType& val) const {
			SingleData<DataType> result = *this;     // Make a copy of myself.  Same as MyClass result(*this);
			result.d1 *= val;       // Use *= to multiply other to the copy.
			return result;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType>& operator*=(const HardenedData::SingleData<DataType>& val) {
			d1 *= val.d1;
			return *this;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType>& operator*=(const DataType& val) {
			d1 *= val;
			return *this;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType> operator/(const HardenedData::SingleData<DataType>& val) const {
			SingleData<DataType> result = *this;     // Make a copy of myself.  Same as MyClass result(*this);
			result.d1 /= val.d1;       // Use /= to divide other to the copy.
			return result;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType> operator/(const DataType& val) const {
			SingleData<DataType> result = *this;     // Make a copy of myself.  Same as MyClass result(*this);
			result.d1 /= val;       // Use /= to divide other to the copy.
			return result;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType>& operator/=(const HardenedData::SingleData<DataType>& val) {
			d1 /= val.d1;
			return *this;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType>& operator/=(const DataType& val) {
			d1 /= val;
			return *this;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType> operator|(const HardenedData::SingleData<DataType>& val) const {
			SingleData<DataType> result = *this;     // Make a copy of myself.  Same as MyClass result(*this);
			result.d1 != val.d1;       // Use *= to multiply other to the copy.
			return result;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType> operator|(const DataType& val) const {
			SingleData<DataType> result = *this;     // Make a copy of myself.  Same as MyClass result(*this);
			result.d1 != val;       // Use *= to multiply other to the copy.
			return result;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType>& operator|=(const DataType& val) {
			d1 |= val;
			return *this;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType>& operator|=(const HardenedData::SingleData<DataType>& val) {
			d1 |= val.d1;
			return *this;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType> operator&(const HardenedData::SingleData<DataType>& val) const {
			SingleData<DataType> result = *this;     // Make a copy of myself.  Same as MyClass result(*this);
			result.d1 &= val.d1;       // Use *= to multiply other to the copy.
			return result;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType> operator&(const DataType& val) const {
			SingleData<DataType> result = *this;     // Make a copy of myself.  Same as MyClass result(*this);
			result.d1 &= val;       // Use *= to multiply other to the copy.
			return result;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType>& operator&=(const DataType& val) {
			d1 &= val;
			return *this;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType>& operator&=(const HardenedData::SingleData<DataType>& val) {
			d1 &= val.d1;
			return *this;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType> operator^(const HardenedData::SingleData<DataType>& val) const {
			SingleData<DataType> result = *this;     // Make a copy of myself.  Same as MyClass result(*this);
			result.d1 ^= val.d1;       // Use *= to multiply other to the copy.
			return result;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType> operator^(const DataType& val) const {
			SingleData<DataType> result = *this;     // Make a copy of myself.  Same as MyClass result(*this);
			result.d1 ^= val;       // Use *= to multiply other to the copy.
			return result;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType>& operator^=(const DataType& val) {
			d1 ^= val;
			return *this;  // Return a reference to myself
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType>& operator^=(const HardenedData::SingleData<DataType>& val) {
			d1 ^= val.d1;
			return *this;  // Return a reference to myself
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType> operator~() const {
			SingleData<DataType> result;
			result.d1 = ~ d1;       // Use -= to add other to the copy.
			return result;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType> operator>>(const int& val) const {
			SingleData<DataType> result;
			result.d1 = d1 >> val;
			return result;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType> operator<<(const int& val) const {
			SingleData<DataType> result;
			result.d1 = d1 << val;
			return result;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType>& operator>>=(const int& val) {
			d1 >>= val;
			return *this;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::SingleData<DataType>& operator<<=(const int& val) {
			d1 <<= val;
			return *this;
		}


		/// <summary>
		/// Overwrites the logic operator a == b when a is redundant SingleData<DataType> while b is DataType.
		/// It first votes a and compares the result against b and returns result as a bool.
		/// Neither variable a nor b is affected.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		bool operator==(const DataType& val) const {
			return ((DataType) *this == val);
		}


		/// <summary>
		/// Overwrites the logic operator a == b when both a and b are redundant SingleData<DataType>.
		/// It first votes a and b separately, then it compares voting result and returns comparison result as a bool.
		/// Neither variable a nor b is affected.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		bool operator==(const HardenedData::SingleData<DataType>& val) const {
			return ((DataType) *this == (DataType) val);
		}


		/// <summary>
		/// Overwrites the logic operator a != b when a is redundant SingleData<DataType> while b is DataType.
		/// It first votes a and compares the result against b and returns result as a bool.
		/// Neither variable a nor b is affected.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		bool operator!=(const DataType& val) const {
			return ((DataType) *this != val);
		}


		/// <summary>
		/// Overwrites the logic operator a != b when both a and b are redundant SingleData<DataType>.
		/// It first votes a and b separately, then it compares voting result and returns comparison result as a bool.
		/// Neither variable a nor b is affected.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		bool operator!=(const HardenedData::SingleData<DataType>& val) const {
			return ((DataType) *this != (DataType) val);
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		DataType sync() {
			return d1;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		bool check() const {
			return true;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		DataType vote() const {
			return (DataType) (*this);
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		bool voteBit(DataType mask) const {
			return (d1&mask);
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		bool agree() const {
			return check();
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		bool agreeBit(DataType mask) const {
			return true;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::WHO disagree() const {
			return NONE;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::WHO disagreeBit(DataType mask) const {
			return NONE;
		}


		/// <summary>
		/// Returns first replica of the data.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		DataType& a() {
			return d1;
		}


		/// <summary>
		/// Returns either:
		/// first replica of the data, if who==AA;
		/// first replica of the data, for any other value of who; 
		///  
		///  
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		DataType& x(const HardenedData::WHO who) {
			switch (who){
			case AA: return d1; break;
			default: return d1; break;
			}
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		void set(const HardenedData::SingleData<DataType>& mask) {
			(*this) |= mask;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		void set(const DataType& mask) {
			(*this) |= mask;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		void reset(const HardenedData::SingleData<DataType>& mask) {
			(*this) &= ~mask;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		void reset(const DataType& mask) {
			(*this) &= ~mask;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		void write(const HardenedData::SingleData<DataType>& val) {
			(*this) =val;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		void write(const DataType& val) {
			(*this) =val;
		}


		/// <summary>
		/// Writes partially, in the three replicas of data stored in the class, the value of the argument data, but only in all the bits which are set to 1 in the argument mask, while the other bits are not affected.
		/// In practice:
		///         TripleData<DataType> a, data, mask;
		///         a = 0x1357;
		///         data = 0xFF00;
		///         mask = 0x0FF0;
		///         a.write(data, mask);
		/// leaves first nibble (1) and last nibble (7) unchanged, then writes F0 in the second and third nibble.
		/// So, at the end, a will contain the value 0x1F07.
		/// This operation is a combination of the operator operator=(val : TripleData  &) : TripleData  &,  operator operator&(val : TripleData  &) : TripleData  &,,  operator operator|(val : TripleData  &) : TripleData  &  and operator operator~() : TripleData  &. For more details and side effects of this operation, see documentation of these operators..
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		void write(HardenedData::SingleData<DataType>& data, HardenedData::SingleData<DataType>& mask) {
			(*this) = (data & mask) | ((*this) & ~mask);
		}


		/// <summary>
		/// Writes partially, in the three replicas of data stored in the class, the value of the argument data, but only in all the bits which are set to 1 in the argument mask, while the other bits are not affected.
		/// In practice:
		///         TripleData<DataType> a,;
		///         DataType data, mask;
		///         a = 0x1357;
		///         data = 0xFF00;
		///         mask = 0x0FF0;
		///         a.write(data, mask);
		/// leaves first nibble (1) and last nibble (7) unchanged, then writes F0 in the second and third nibble.
		/// So, at the end, a will contain the value 0x1F07.
		/// This operation is a combination of the operator operator=(val : DataType) : TripleData  &,  operator operator&(val : DataType &) : TripleData  &, operator operator|(val : DataType &) : TripleData  & and ~. For more details and side effects of these operations, see documentation of these operators.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		void write(DataType& data, DataType& mask) {
			(*this) = (data & mask) | ((*this) & (DataType)(~mask));
		}


		/// <summary>
		/// Returns true if and only if the bits which are set in data are set also in the data stored in the class.
		/// In practice:
		///         SingleData<DataType> a, data;
		///         a = 0x1F07;
		///         data = 0x1F00;
		///         a.test(data);
		/// will return true, as all the bits which are set in data are also set in a, while if:
		///         a = 0x15D7;
		///         data = 0xFF00;
		///         a.test(data);
		/// will return false, as some of the bits which are set in data are NOT set in a.
		/// This operation is equivalent to test(data, data).
		/// For more details and side effects of this operation, see documentation of these operators.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		bool test(const HardenedData::SingleData<DataType>& data) const {
			return (*this).test(data, data);
		}


		/// <summary>
		/// Returns true if and only if the bits which are set in data are set also in the data stored in the class.
		/// In practice:
		///         SingleData<DataType> a, data;
		///         a = 0x1F07;
		///         data = 0x1F00;
		///         a.test(data);
		/// will return true, as all the bits which are set in data are also set in a, while if:
		///         a = 0x15D7;
		///         data = 0xFF00;
		///         a.test(data);
		/// will return false, as some of the bits which are set in data are NOT set in a.
		/// This operation is equivalent to test(data, data).
		/// For more details and side effects of this operation, see documentation of these operators.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		bool test(const DataType& data) const {
			return (*this).test(data, data);
		}


		/// <summary>
		/// Returns true if and only if ALL bits of data stored in the class, corresponding to bits set to 1 in argument mask are identical to data.
		/// In practice:
		///         TripleData<DataType> a, data, mask;
		///         a = 0x1F07;
		///         data = 0xFF00;
		///         mask = 0x0FF0;
		///         a.test(data, mask);
		/// will return true, as all the bits in the second and third nibble (namely, all those bits where mask is 1) of a are identical to data, while if:
		///         a = 0x15D7;
		///         data = 0xFF00;
		///         mask = 0x0FF0;
		///         a.test(data, mask);
		/// will return false, as some of the bits in the second and third nibble (namely, all those bits where mask is 1) of a are NOT identical to data.
		/// This operation is a combination of the operator operator=(val : TripleData  &) : TripleData  &,  operator operator&(val : TripleData  &) : TripleData  &,,  operator operator|(val : TripleData  &) : TripleData  &  and operator operator~() : TripleData  &. For more details and side effects of this operation, see documentation of these operators.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		bool test(const HardenedData::SingleData<DataType>& data, const HardenedData::SingleData<DataType>& mask) const {
			return (((*this) & mask) == (data & mask));
		}


		/// <summary>
		/// Returns true if and only if ALL bits of data stored in the class, corresponding to bits set to 1 in argument mask are identical to data.
		/// In practice:
		///         TripleData<DataType> a;
		///         DataType data, mask;
		///         a = 0x1F07;
		///         data = 0xFF00;
		///         mask = 0x0FF0;
		///         a.test(data, mask);
		/// will return true, as all the bits in the second and third nibble (namely, all those bits where mask is 1) of a are identical to data, while if:
		///         a = 0x15D7;
		///         data = 0xFF00;
		///         mask = 0x0FF0;
		///         a.test(data, mask);
		/// will return false, as some of the bits in the second and third nibble (namely, all those bits where mask is 1) of a are NOT identical to data.
		/// This operation is a combination of the operator operator=(val : DataType) : TripleData  &,  operator operator&(val : DataType &) : TripleData  &, operator operator|(val : DataType &) : TripleData  & and ~. For more details and side effects of these operations, see documentation of these operators.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		bool test(const DataType& data, const DataType& mask) const {
			return (((*this) & mask) == (DataType)(data & mask));
		}


		/// <summary>
		/// Returns true if and only if ALL bits of data stored in the class are identical to data.
		/// In practice:
		///         TripleData<DataType> a, data;
		///         a = 0x1F07;
		///         data = 0x1F07;
		///         a.test(data);
		/// will return true, as all the bits of a are identical to data, while if:
		///         a = 0x15D7;
		///         data = 0xFF00;
		///         a.test(data);
		/// will return false, as some of the bits of a are NOT identical to data.
		/// This operation is a combination of the operator operator=(val : TripleData  &) : TripleData  &,  operator operator&(val : TripleData  &) : TripleData  &,,  operator operator|(val : TripleData  &) : TripleData  &  and operator operator~() : TripleData  &. For more details and side effects of this operation, see documentation of these operators.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		bool testExact(const HardenedData::SingleData<DataType>& data) const {
			return ((*this) == data);
		}


		/// <summary>
		/// Returns true if and only if ALL bits of data stored in the class are identical to data.
		/// In practice:
		///         TripleData<DataType> a;
		///         a = 0x1F07;
		///         data = 0x1F07;
		///         a.test(data);
		/// will return true, as all the bits of a are identical to data, while if:
		///         a = 0x15D7;
		///         data = 0xFF00;
		///         a.test(data);
		/// will return false, as some of the bits of a are NOT identical to data.
		/// This operation is a combination of the operator operator=(val : DataType) : TripleData  &,  operator operator&(val : DataType &) : TripleData  &, operator operator|(val : DataType &) : TripleData  & and ~. For more details and side effects of these operations, see documentation of these operators.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		bool testExact(const DataType& data) const {
			return ((*this) == data);
		}


 
		public: void swap(HardenedData::SingleData<DataType>& data) {
			DataType tmp;
			tmp = d1;
			d1 = data.d1;
			data.d1 = tmp;
		}

	};
}
#endif
